//--------------------------------------------------------------------------------------
// File: main.fx
//
// The effect file for the main sample.  
// 
// Copyright (c) Takashi Imagire. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float	 g_fTime;					// App's time in seconds
float	 g_fRadius = 5.0;			// 
float4x4 g_mWorld;					// World matrix for object
float4x4 g_mWorldViewProjection;	// World * View * Projection matrix
float4x4 g_mShadowViewProjection;
float4x4 g_mShadowMatrix;
float4   g_Offset[12];
float4   g_vLightDir;

//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
texture Texture;
sampler Sampler = 
sampler_state
{
    Texture = <Texture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture ShadowTex;
sampler ShadowSampler = 
sampler_state
{
    Texture = <ShadowTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};

//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position  : POSITION;
    float4 Pos       : TEXCOORD0;
};



//-----------------------------------------------------------------------------
// Name: CreateShadowMapVS
// Type: Vertex shader                                      
// Desc: standard transform and lighting.
//-----------------------------------------------------------------------------
VS_OUTPUT CreateShadowMapVS( float4 vPos : POSITION, 
                    float3 vNormal : NORMAL,
                    float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;
    float4 pos;
  
    // tranform vertex position into screen space
    pos = mul(vPos, g_mWorldViewProjection );
    Output.Position = pos;
    
    // ˉeԂ̍WleNX`Wɂēn
    Output.Pos      = pos;
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: SimplePS
// Type: Pixel shader
// Desc: This shader simply outputs the pixel's color 
//-----------------------------------------------------------------------------
float4 CreateShadowMapPS( VS_OUTPUT In ) : COLOR
{ 
    return In.Pos.z / In.Pos.w;
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique CreateShadowMap
{
    pass P0
    {
        VertexShader = compile vs_1_1 CreateShadowMapVS();
        PixelShader  = compile ps_2_0 CreateShadowMapPS();
    }
}



//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT_SHADOW
{
    float4 Position  : POSITION;
    float4 Diffuse   : COLOR0;
    float4 Pos       : TEXCOORD0;
    float4 TextureUV : TEXCOORD1;
    float4 ShadowMap : TEXCOORD2;
};


//-----------------------------------------------------------------------------
// Name: ShadowMapVS
// Type: Vertex shader                                      
// Desc: standard transform and lighting.
//-----------------------------------------------------------------------------
VS_OUTPUT_SHADOW ShadowMapVS( float4 vPos : POSITION, 
                    float3 vNormal : NORMAL,
                    float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT_SHADOW Output;
    float4 pos;
  
    // tranform vertex position into screen space
    Output.Position  = mul(vPos, g_mWorldViewProjection );
    
    // [JWnł̌vZ
    Output.Diffuse = max( dot(g_vLightDir.xyz, vNormal), 0);
    
    // zJƂ݂ȂƂ̎ˉeԂւ̃}bsO
    Output.Pos       = mul(vPos, g_mShadowViewProjection );
    
    // ɃoCAXςāAeNX`̍WnɑΉ
    Output.ShadowMap = mul(vPos, g_mShadowMatrix );

    // eNX`WPɃRs[
    Output.TextureUV.xy = vTexCoord0;
    Output.TextureUV.wz = 0;
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: ShadowMapPS
// Type: Pixel shader
// Desc: PȃVhE}bsO
//-----------------------------------------------------------------------------
float4 ShadowMapPS( VS_OUTPUT_SHADOW In ) : COLOR
{ 
	float shadow_map = tex2Dproj( ShadowSampler, In.ShadowMap ).x;
	float depth = In.Pos.z / In.Pos.w;
	float shade = ( shadow_map < depth - 0.003 ) ? 0 : 1 ;
	
	const float ambient = 0.3;
	
    return tex2D( Sampler, In.TextureUV )
			 * ((1.0-ambient) * shade * In.Diffuse + ambient);
}

//-----------------------------------------------------------------------------
// Name: ShadowMap12PS
// Type: Pixel shader
// Desc: KEX򂵂p\tgVhE
//-----------------------------------------------------------------------------
float4 ShadowMap12PS( VS_OUTPUT_SHADOW In ) : COLOR
{
	float2 t = In.ShadowMap.xy / In.ShadowMap.w;
	float2 t0 = t + g_Offset[ 0].xy;
	float2 t1 = t + g_Offset[ 1].xy;
	float2 t2 = t + g_Offset[ 2].xy;
	float2 t3 = t + g_Offset[ 3].xy;
	float2 t4 = t + g_Offset[ 4].xy;
	float2 t5 = t + g_Offset[ 5].xy;
	float2 t6 = t + g_Offset[ 6].xy;
	float2 t7 = t + g_Offset[ 7].xy;
	float2 t8 = t + g_Offset[ 8].xy;
	float2 t9 = t + g_Offset[ 9].xy;
	float2 ta = t + g_Offset[10].xy;
	float2 tb = t + g_Offset[11].xy;
	
	float shadow_map0 = tex2D( ShadowSampler, t0 ).x;
	float shadow_map1 = tex2D( ShadowSampler, t1 ).x;
	float shadow_map2 = tex2D( ShadowSampler, t2 ).x;
	float shadow_map3 = tex2D( ShadowSampler, t3 ).x;
	float shadow_map4 = tex2D( ShadowSampler, t4 ).x;
	float shadow_map5 = tex2D( ShadowSampler, t5 ).x;
	float shadow_map6 = tex2D( ShadowSampler, t6 ).x;
	float shadow_map7 = tex2D( ShadowSampler, t7 ).x;
	float shadow_map8 = tex2D( ShadowSampler, t8 ).x;
	float shadow_map9 = tex2D( ShadowSampler, t9 ).x;
	float shadow_mapa = tex2D( ShadowSampler, ta ).x;
	float shadow_mapb = tex2D( ShadowSampler, tb ).x;
	
	float depth = In.Pos.z / In.Pos.w - 0.003;
	float shade = (( shadow_map0 < depth ) ? 0 : 1)
	            + (( shadow_map1 < depth ) ? 0 : 1)
	            + (( shadow_map2 < depth ) ? 0 : 1)
	            + (( shadow_map3 < depth ) ? 0 : 1)
	            + (( shadow_map4 < depth ) ? 0 : 1)
	            + (( shadow_map5 < depth ) ? 0 : 1)
	            + (( shadow_map6 < depth ) ? 0 : 1)
	            + (( shadow_map7 < depth ) ? 0 : 1)
	            + (( shadow_map8 < depth ) ? 0 : 1)
	            + (( shadow_map9 < depth ) ? 0 : 1)
	            + (( shadow_mapa < depth ) ? 0 : 1)
	            + (( shadow_mapb < depth ) ? 0 : 1);
	            
	const float ambient = 0.3;
	
    return tex2D( Sampler, In.TextureUV )
			 * (((1.0-ambient)/12.0) * shade * In.Diffuse + ambient);
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique ShadowMap
{
    pass P0
    {
        VertexShader = compile vs_1_1 ShadowMapVS();
        PixelShader  = compile ps_2_0 ShadowMap12PS();
    }
}





